#include <exec/exec.h>
#include <devices/timer.h>

#include <proto/exec.h>
#include <proto/timer.h>

#include "Timing.h"

struct timerequest *_OpenTimerDevice(struct ExecBase *SysBase, ULONG unitNum) {
	struct MsgPort *timerReplyPort = CreateMsgPort();
	struct timerequest *timerReq = (struct timerequest *) CreateIORequest(timerReplyPort, sizeof(struct timerequest));
	if (NULL == timerReq || 0 != OpenDevice("timer.device", unitNum, (struct IORequest *) timerReq, 0)) {
		DeleteIORequest((struct IORequest *) timerReq);
		DeleteMsgPort(timerReplyPort);
		timerReq = NULL;
	}
	return timerReq;
}

void _CloseTimerDevice(struct ExecBase *SysBase, struct timerequest *timerReq) {
	if (NULL != timerReq) {
		CloseDevice((struct IORequest *) timerReq);
		struct MsgPort *timerReplyPort = timerReq->tr_node.io_Message.mn_ReplyPort;
		DeleteIORequest((struct timerequest *) timerReq);
		DeleteMsgPort(timerReplyPort);
	}
}

static uint32_t GetEClockFreq(struct Library *TimerBase) {
	struct EClockVal dummyEClockVal;
	return ReadEClock(&dummyEClockVal);
}

uint64_t _GetEClocks(struct Library *TimerBase) {
	struct EClockVal eClockVal;
	ReadEClock(&eClockVal);
	return (uint64_t) eClockVal.ev_hi << 32 | eClockVal.ev_lo;
}

struct timeval _EClocksToTimeval(struct Library *TimerBase, const uint64_t eClocks) {
	const uint32_t eClockFreq = GetEClockFreq(TimerBase);
	return (struct timeval) {
		.tv_secs = eClocks / eClockFreq,
		.tv_micro = ((eClocks % eClockFreq) * 999999) / eClockFreq };
}
